import csv
import argparse
import json
import pkg_resources


def read_csv(path, fieldnames):
    data = []
    with open(path) as csvfile:
        reader = csv.DictReader(csvfile, fieldnames=fieldnames)
        for row in reader:
            data.append(row)
    return data


def get_headers(api_key, processor_name):
    return {
        'x-api-key': api_key,
        'Accept': 'application/json',
        'Content-Type': 'application/json',
        'Processor-Name': processor_name,
        'Processor-Version': pkg_resources.get_distribution("rhumbix_csv_uploader").version
    }


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("csv_path")
    parser.add_argument('company_key', nargs='?')
    parser.add_argument('api_key', nargs='?')
    args = parser.parse_args()
    if args.company_key is None or args.api_key is None:
        config_dict = json.load(open('config.json'))
        args.company_key = config_dict["company_key"]
        args.api_key = config_dict["api_key"]
    return args



import unittest
import rhumbix_csv_uploader.project_processor as processor
from rhumbix_csv_uploader.project_processor import get_batch_status, post_data, read_project_csv, transform_values
from rhumbix_csv_uploader.test.processor_test_config import delay_between_requests, company_key, api_key, api_url


project_test_csv = "rhumbix_csv_uploader/test/exampleCSVs/rhumbix_project.csv"
processor.company_key = company_key
processor.api_key = api_key
processor.api_url = api_url


class TestTransformValues(unittest.TestCase):
    def test_transform_values(self):
        data = read_project_csv(project_test_csv)
        data = transform_values(data)
        assert(isinstance(data[0]["is_active"], bool))


class TestPostData(unittest.TestCase):
    def test_post_then_batch_status(self):
        data = read_project_csv(project_test_csv)
        data = transform_values(data)
        result = post_data(data)
        assert(result.status_code == 200)
        body = result.json()
        assert('batch_key' in body)
        batch_key = body["batch_key"]
        import time
        time.sleep(delay_between_requests)
        response = get_batch_status(batch_key)
        assert(response.status_code == 200), "batch_key=%s, body=%s" % (
            batch_key, response.json())
        if(response.status_code == 200):
            body = response.json()
            assert(body["status"] in ["ACCEPTED", "PROCESSING", "COMPLETE"]), "body=%s" % body
            assert(len(body["errors"].items()) == 0)



import requests
import logging
from rhumbix_csv_uploader.processor_utils import read_csv, parse_arguments, get_headers


logging.basicConfig(level=logging.INFO)
fieldnames = ["job_number", "Customer Number - unused", "name", "is_active", "address"]
company_key = None
api_key = None
api_url = None
PROCESSOR_NAME = "project_processor"


def read_project_csv(path):
    return read_csv(path, fieldnames)


def transform_values(data):
    """
    Transforms the csv values into the Rhumbix API format
    """
    for d in data:
        d["is_active"] = d["is_active"] == "A"
    return data


def post_data(data):
    url = 'https://async-{}/V1_0_1/{}/batch/project/import'.format(
        api_url, company_key)
    result = requests.post(url, headers=get_headers(api_key, PROCESSOR_NAME), json=data)
    return result


def get_batch_status(batch_id):
    url = 'https://async-{}/V1_0_1/{}/batch/project/import/{}/status'.format(
        api_url, company_key, batch_id)
    return requests.get(url, headers=get_headers(api_key, PROCESSOR_NAME))


def process_csv(path, _company_key, _api_key, _api_url):
    global company_key
    global api_key
    global api_url
    company_key = _company_key
    api_key = _api_key
    api_url = _api_url
    data = read_project_csv(path)
    data = transform_values(data)
    return post_data(data)


if __name__ == "__main__":
    args = parse_arguments()
    result = process_csv(args.csv_path, args.company_key, args.api_key, args.api_url)
    log_function = logging.info if 200 <= result.status_code <= 299 else logging.warning
    if log_function is logging.warning:
        log_function("Something went wrong with uploading csv data")
    log_function(result.status_code)
    log_function(result.json())



import pypandoc

description = pypandoc.convert('README.md', 'rst')
description = description.replace("\r", "")  # YOU  NEED THIS LINE

with open("README.rst", "w") as text_file:
    text_file.write(description)



import argparse
import json
import glob
import logging
import time
import os
from os.path import dirname, abspath
import rhumbix_csv_uploader.employee_processor as employee_processor
import rhumbix_csv_uploader.project_processor as project_processor
import rhumbix_csv_uploader.cost_code_processor as cost_code_processor


employee_filename_pattern = "rhumbix_employee*.csv"
project_filename_pattern = "rhumbix_project*.csv"
cost_code_filename_pattern = "rhumbix_cost_code*.csv"
DEFAULT_API_URL = "api.rhumbix.com"
BATCH_LOOKUP_CYCLE_DELAY_SECONDS = 2
TWO_WEEKS_IN_SECONDS = 2 * 7 * 24 * 60 * 60


def clean_processed_folder(base_path):
    logging.info("Cleaning processed directory of expired files")
    processed_path = os.path.join(base_path, "processed")
    processed_path = os.path.join(processed_path, '')
    if not os.path.exists(processed_path):
        return
    paths = glob.glob("%s%s" % (processed_path, "*.csv"))
    for path in paths:
        if time.time() - os.path.getmtime(path) > TWO_WEEKS_IN_SECONDS:
            logging.info("Removing %s as it has expired", path)
            os.remove(path)


def move_file_to_processed(path):
    if os.path.exists(path):
        parent_path = (dirname(abspath(path)))
        processed_path = os.path.join(parent_path, "processed")
        if not os.path.exists(processed_path):
            # Create the processed directory
            os.makedirs(processed_path)
        processed_path = os.path.join(processed_path, os.path.basename(path))
        if os.path.exists(processed_path):
            logging.warning("Overwriting %s in processed directory!" % processed_path)
        os.rename(path, processed_path)
    else:
        logging.warning("move_file_to_processed received nonexistent file: %s" % path)


def main():
    global employee_filename_pattern
    global project_filename_pattern
    global cost_code_filename_pattern
    parser = argparse.ArgumentParser()
    parser.add_argument("csvs_directory")
    parser.add_argument('company_key', nargs='?')
    parser.add_argument('api_key', nargs='?')
    parser.add_argument('api_url', nargs='?')
    args = parser.parse_args()

    config_dict = None
    if os.path.exists('config.json'):
        config_dict = json.load(open('config.json'))
        for name in ["company_key", "api_key", "api_url"]:
            if not getattr(args, name, None):
                setattr(args, name, config_dict.get(name, None))

    for name in ["company_key", "api_key"]:
        if not getattr(args, name, None):
            logging.error("{} is not set. It can be passed as a command line argument or in config.json".format(name))

    if args.api_url is None:
        args.api_url = DEFAULT_API_URL

    # Load custom file name patterns
    if config_dict is not None:
        if "employee_filename_pattern" in config_dict:
            employee_filename_pattern = config_dict["employee_filename_pattern"]
        if "project_filename_pattern" in config_dict:
            project_filename_pattern = config_dict["project_filename_pattern"]
        if "cost_code_filename_pattern" in config_dict:
            cost_code_filename_pattern = config_dict["cost_code_filename_pattern"]

    args.csvs_directory = os.path.join(args.csvs_directory, '')

    logging.info("Processing directory %s with company_key=%s and api_key=%s and api_url=%s" %
        (args.csvs_directory, args.company_key, args.api_key, args.api_url))

    clean_processed_folder(args.csvs_directory)

    employee_paths = glob.glob("%s%s" % (args.csvs_directory, employee_filename_pattern))
    logging.info("Employee files=%s" % employee_paths)

    project_paths = glob.glob("%s%s" % (args.csvs_directory, project_filename_pattern))
    logging.info("Project files=%s" % project_paths)

    cost_code_paths = glob.glob("%s%s" % (args.csvs_directory, cost_code_filename_pattern))
    logging.info("Cost Code Files=%s" % cost_code_paths)

    employee_batches = []
    for path in employee_paths:
        logging.info("Processing employee file: %s" % path)

        result = employee_processor.process_csv(path, args.company_key, args.api_key, args.api_url)
        if 'batch_key' not in result.json():
            logging.error("Request failed! Here is the body:\n%s" % result.json())
            continue

        batch_key = result.json()["batch_key"]
        logging.info("Finished file:%s with batch id: %s" % (path, batch_key))
        employee_batches.append({"path": path, "batch_key": batch_key})

    project_batches = []
    for path in project_paths:
        logging.info("Proccessing project file: %s" % path)

        result = project_processor.process_csv(path, args.company_key, args.api_key, args.api_url)
        if 'batch_key' not in result.json():
            logging.error("Request failed! Here is the body:\n%s" % result.json())
            continue

        batch_key = result.json()["batch_key"]
        logging.info("Finished file:%s with batch id: %s" % (path, batch_key))
        project_batches.append({"path": path, "batch_key": batch_key})

    cost_code_batches = []
    for path in cost_code_paths:
        logging.info("Processing cost code file: %s" % path)

        result = cost_code_processor.process_csv(path, args.company_key, args.api_key, args.api_url)
        if 'batch_key' not in result.json():
            logging.error("Request failed! Here is the body:\n%s" % result.json())
            continue

        batch_key = result.json()["batch_key"]
        logging.info("Finished file:%s with batch id: %s" % (path, batch_key))
        cost_code_batches.append({"path": path, "batch_key": batch_key})

    logging.info("All csv files uploaded. Monitoring batch status...")
    # Wait for all the batches to get done. The [:] makes a temporary copy as the list is mutated
    while len(employee_batches) + len(project_batches) + len(cost_code_batches) > 0:
        for batch in employee_batches[:]:
            batch_key = batch["batch_key"]
            result = employee_processor.get_batch_status(batch_key)
            body = result.json()
            logging.info("Batch id %s has status: %d %s" % (batch_key, result.status_code, body["status"]))
            if result.status_code == 200 and body["status"] == "COMPLETE":
                move_file_to_processed(batch["path"])
                employee_batches.remove(batch)

        for batch in project_batches[:]:
            batch_key = batch["batch_key"]
            result = project_processor.get_batch_status(batch_key)
            body = result.json()
            logging.info("Batch id %s has status: %d %s" % (batch_key, result.status_code, body["status"]))
            if result.status_code == 200 and body["status"] == "COMPLETE":
                move_file_to_processed(batch["path"])
                project_batches.remove(batch)

        for batch in cost_code_batches[:]:
            batch_key = batch["batch_key"]
            result = cost_code_processor.get_batch_status(batch_key)
            body = result.json()
            logging.info("Batch id %s has status: %d %s" % (batch_key, result.status_code, body["status"]))
            if result.status_code == 200 and body["status"] == "COMPLETE":
                move_file_to_processed(batch["path"])
                cost_code_batches.remove(batch)

        time.sleep(BATCH_LOOKUP_CYCLE_DELAY_SECONDS)

    logging.info("All batches have completed processing")


if __name__ == "__main__":
    main()
